import openai
import json


class GPTAssistant:
    def __init__(self, api_key, version: int = 3, language: str = "en", max_length: int = 300, is_html: bool = True, allowed_tags: list = []):
        openai.api_key = api_key
        self.messages = []
        self.version = version
        self.language = language
        self.max_length = max_length
        self.keys = ["request", "body"]

        self.is_html = is_html
        self.allowed_tags = []

        if is_html:
            self.allowed_tags = ['p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
                                 'ul', 'ol', 'li', 'a', 'br', 'b', 'i', 'strong', 'em', 'u', 'code', 'hr', 'blockquote', 'div']

            if allowed_tags:
                self.allowed_tags = allowed_tags

        self.set_rules()

    def set_rules(self):

        self.add_role('system', f"Your language (iso): {self.language}")

        self.add_role(
            'system', f"Use max_length: {self.max_length} in json.body"
        )

        json_format = json.dumps({key: "str" for key in self.keys})
        self.add_role(
            'system', f"Return response in JSON format with double quotes. Example: {json_format}"
        )

        if self.is_html and self.allowed_tags:
            tags = ", ".join(f"<{tag}>" for tag in self.allowed_tags)
            self.add_role(
                'system', f"In json.body use html tags: {tags}."
            )

    def response_decoder(self, response):
        try:
            for key in self.keys:
                response = response.replace(f"'{key}':", f'"{key}":')

            res = json.loads(response)
            return res["body"]
        except Exception as e:
            print('@' * 30)
            print(f"Error in response_decoder: {e}")
            print(f"Response: {response}")
            # raise Exception(f"Error in response_decoder: {e}")
            return response

    def add_role(self, role, content):
        if isinstance(content, str):
            self.messages.append({"role": role, "content": content})

    def clear_messages(self):
        self.messages = []

    def generate_response(self, temperature: float = 0.7):

        # print(self.messages)

        try:
            model_version = "gpt-3.5-turbo" if self.version == 3 else "gpt-4"
            max_tokens_per_request = 1000  # Max tokens per request

            response_parts = []

            # Split messages into chunks so that each chunk fits within the token limit
            for i in range(0, len(self.messages), max_tokens_per_request):
                chunk_messages = self.messages[i:i + max_tokens_per_request]

                response = openai.ChatCompletion.create(
                    model=model_version,
                    messages=chunk_messages,
                    max_tokens=max_tokens_per_request,
                    temperature=temperature,
                    n=1,
                    stop=None,
                )

                response_content = response.choices[0].message["content"]
                response_parts.append(response_content)

            # Combine all response parts into a single string
            full_response = "".join(response_parts)

            # Clear messages for next request
            self.clear_messages()

            return self.response_decoder(full_response)

        except openai.error.OpenAIError as e:
            raise Exception(f"OpenAI Error: {str(e)}")

    def set_words(self, content: str):
        self.messages.append(
            {"role": "user", "content": content}
        )


if __name__ == "__main__":
    # Set your OpenAI API key
    CHATGPT_API_KEY = "***"

    # Create an instance of the GPTAssistant class
    assistant = GPTAssistant(CHATGPT_API_KEY, language="ru", max_length=300)

    words_to_generate = "Where to buy a house in the city of London?"

    assistant.set_words(words_to_generate)

    assistant.add_role(
        'assistant', "You're a real estate agent and you're writing a description for a property.")

    response = assistant.generate_response()

    print(response)



from django.db import models
from django_ckeditor_5.widgets import CKEditor5Widget
from django.conf import settings
from django.db import models
from django import forms

from .ai import GPTAssistant


class GptGenerationError(Exception):
    pass


class DjapgoGptField(models.Field):
    def __init__(self, *args,
                 type="text",
                 config_name="default",
                 gpt_role="",
                 gpt_content_length=300,
                 field_instruction_name="gpt_instruction",
                 allowed_tags=['p', 'br', 'b', 'strong', 'i', 'em', 'u',
                               'ul', 'ol', 'li', 'a', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6'],
                 ** kwargs
                 ):

        if type not in ["text", "textarea", "html"]:
            raise Exception('type must be one of "text", "textarea", "html"')

        self.type = type
        self.config_name = config_name
        self.gpt_role = gpt_role
        self.gpt_content_length = gpt_content_length
        self.field_instruction_name = field_instruction_name
        self.allowed_tags = allowed_tags

        super().__init__(*args, **kwargs)

    def get_internal_type(self):
        return "TextField"

    def formfield(self, **kwargs):

        if self.type == "textarea":
            return super().formfield(
                **{
                    "max_length": self.max_length,
                    **({} if self.choices is not None else {
                        "widget": forms.Textarea(attrs={
                            'cols': 200, 'rows': 10,
                        }),
                    }),
                    **kwargs,
                }
            )

        if self.type == "html":
            return super().formfield(
                **{
                    "max_length": self.max_length,
                    ** ({"widget": CKEditor5Widget(config_name=self.config_name)}),
                    **kwargs,
                }
            )

        return super().formfield(
            **{
                "max_length": self.max_length,
                **({} if self.choices is not None else {
                    "widget": forms.TextInput(attrs={'size': '100%'})
                }),
                **kwargs,
            }
        )

    def pre_save(self, model_instance, add):
        if not hasattr(model_instance, self.field_instruction_name):
            raise Exception('field_instruction_name field is required')

        gpt_instruction = getattr(model_instance, self.field_instruction_name)
        gpt_description = getattr(model_instance, self.attname)

        old_gpt_instruction = model_instance.__class__._default_manager.filter(
            pk=model_instance.pk).values_list(self.field_instruction_name, flat=True).first()

        if gpt_instruction and old_gpt_instruction != gpt_instruction:

            try:
                is_html = self.type == "html"
                gpt_description_response = get_content(
                    gpt_request=gpt_instruction,
                    gpt_pre_content=gpt_description,
                    gpt_content_length=self.gpt_content_length,
                    gpt_role=self.gpt_role,
                    allowed_tags=self.allowed_tags,
                    is_html=is_html,
                )
                if gpt_description_response:
                    setattr(model_instance, self.attname,
                            gpt_description_response)

            except GptGenerationError as e:
                GptGenerationError(e)

        setattr(model_instance, self.field_instruction_name, gpt_instruction)

        return super().pre_save(model_instance, add)


def get_content(
    gpt_request,
    gpt_pre_content,
    gpt_content_length,
    gpt_role=None,
    allowed_tags=[],
    is_html=False,
):
    CHATGPT_API_KEY = settings.CHATGPT_API_KEY

    # Create an instance of the GPTAssistant class

    language = 'en'
    gpt_version = 3

    if hasattr(settings, 'DJANGO_GPT_VERSION'):
        v = settings.DJANGO_GPT_VERSION
        if v == 3:
            gpt_version = 3
        elif v == 4:
            gpt_version = 4

    if hasattr(settings, 'DJANGO_GPT_LANGUAGE'):
        language = settings.DJANGO_GPT_LANGUAGE
    elif hasattr(settings, 'LANGUAGE_CODE'):
        language = settings.LANGUAGE_CODE

    assistant = GPTAssistant(
        api_key=CHATGPT_API_KEY,
        version=gpt_version,
        language=language,
        max_length=gpt_content_length,
        is_html=is_html,
        allowed_tags=allowed_tags,
    )

    assistant.add_role('system', gpt_role)

    if gpt_pre_content:
        assistant.add_role("assistant", f"Previous content: {gpt_pre_content}")

    assistant.set_words(gpt_request)

    response = assistant.generate_response()
    return response



from setuptools import setup, find_packages

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name='django-gpt',
    version='1.0.0',
    description='Integrate powerful text generators like GPT-3 and GPT-4 into your Django application for automated text generation based on instructions.',
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author='Mark',
    author_email='markolofsen@gmail.com',
    url='https://github.com/markolofsen/django-gpt',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    packages=find_packages(),
    install_requires=[
        'openai',
        # 'django-ckeditor-5',
    ],
    # package_dir={'': 'py-seo-html'},
    py_modules=['django_gpt'],
)
